@@ -1,27 +1,37 @@
-/* $Id: strformat.cpp 147612 2021-10-18 12:27:53Z bird $ */
+/* $Id: strformat.cpp 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * IPRT - String Formatter.
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -45,6 +55,36 @@
 #include "internal/string.h"
 
 
+/**
+ * Deals with bad pointers.
+ */
+DECLHIDDEN(size_t) rtStrFormatBadPointer(size_t cch, PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, int cchWidth,
+                                         unsigned fFlags, void const *pvStr, char szTmp[64], const char *pszTag, int cchTag)
+{
+    static char const s_szNull[] = "<NULL>";
+    int               cchStr     = !pvStr ? sizeof(s_szNull) - 1 : 1 + sizeof(void *) * 2 + cchTag + 1;
+
+    if (!(fFlags & RTSTR_F_LEFT))
+        while (--cchWidth >= cchStr)
+            cch += pfnOutput(pvArgOutput, " ", 1);
+
+    cchWidth -= cchStr;
+    if (!pvStr)
+        cch += pfnOutput(pvArgOutput, s_szNull, sizeof(s_szNull) - 1);
+    else
+    {
+        cch += pfnOutput(pvArgOutput, "<", 1);
+        cchStr = RTStrFormatNumber(&szTmp[0], (uintptr_t)pvStr, 16, sizeof(char *) * 2, 0, RTSTR_F_ZEROPAD);
+        cch += pfnOutput(pvArgOutput, szTmp, cchStr);
+        cch += pfnOutput(pvArgOutput, pszTag, cchTag);
+        cch += pfnOutput(pvArgOutput, ">", 1);
+    }
+
+    while (--cchWidth >= 0)
+        cch += pfnOutput(pvArgOutput, " ", 1);
+    return cch;
+}
+
 
 /**
  * Finds the length of a string up to cchMax.
@@ -52,7 +92,7 @@
  * @param     psz     Pointer to string.
  * @param     cchMax  Max length.
  */
-static unsigned _strnlen(const char *psz, unsigned cchMax)
+static unsigned rtStrFormatStrNLen(const char *psz, unsigned cchMax)
 {
     const char *pszC = psz;
 
@@ -69,7 +109,7 @@ static unsigned _strnlen(const char *psz, unsigned cchMax)
  * @param     pwsz    Pointer to string.
  * @param     cchMax  Max length.
  */
-static unsigned _strnlenUtf16(PCRTUTF16 pwsz, unsigned cchMax)
+static unsigned rtStrFormatStrNLenUtf16(PCRTUTF16 pwsz, unsigned cchMax)
 {
 #ifdef IN_RING3
     unsigned cwc = 0;
@@ -100,7 +140,7 @@ static unsigned _strnlenUtf16(PCRTUTF16 pwsz, unsigned cchMax)
  * @param     pusz    Pointer to string.
  * @param     cchMax  Max length.
  */
-static unsigned _strnlenUni(PCRTUNICP pusz, unsigned cchMax)
+static unsigned rtStrFormatStrNLenUni(PCRTUNICP pusz, unsigned cchMax)
 {
     PCRTUNICP   puszC = pusz;
 
@@ -513,83 +553,83 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                         if (chArgSize == 'l')
                         {
                             /* utf-16 -> utf-8 */
-                            int         cchStr;
-                            PCRTUTF16   pwszStr = va_arg(args, PRTUTF16);
-
-                            if (!VALID_PTR(pwszStr))
-                            {
-                                static RTUTF16  s_wszNull[] = {'<', 'N', 'U', 'L', 'L', '>', '\0' };
-                                pwszStr = s_wszNull;
-                            }
-                            cchStr = _strnlenUtf16(pwszStr, (unsigned)cchPrecision);
-                            if (!(fFlags & RTSTR_F_LEFT))
-                                while (--cchWidth >= cchStr)
-                                    cch += pfnOutput(pvArgOutput, " ", 1);
-                            cchWidth -= cchStr;
-                            while (cchStr-- > 0)
+                            PCRTUTF16 pwszStr = va_arg(args, PCRTUTF16);
+                            if (RT_VALID_PTR(pwszStr))
                             {
+                                int cwcStr = rtStrFormatStrNLenUtf16(pwszStr, (unsigned)cchPrecision);
+                                if (!(fFlags & RTSTR_F_LEFT))
+                                    while (--cchWidth >= cwcStr)
+                                        cch += pfnOutput(pvArgOutput, " ", 1);
+                                cchWidth -= cwcStr;
+                                while (cwcStr-- > 0)
+                                {
 /** @todo \#ifndef IN_RC*/
 #ifdef IN_RING3
-                                RTUNICP Cp;
-                                RTUtf16GetCpEx(&pwszStr, &Cp);
-                                char *pszEnd = RTStrPutCp(szTmp, Cp);
-                                *pszEnd = '\0';
-                                cch += pfnOutput(pvArgOutput, szTmp, pszEnd - szTmp);
+                                    RTUNICP Cp;
+                                    RTUtf16GetCpEx(&pwszStr, &Cp);
+                                    char *pszEnd = RTStrPutCp(szTmp, Cp);
+                                    *pszEnd = '\0';
+                                    cch += pfnOutput(pvArgOutput, szTmp, pszEnd - szTmp);
 #else
-                                char ch = (char)*pwszStr++;
-                                cch += pfnOutput(pvArgOutput, &ch, 1);
+                                    char ch = (char)*pwszStr++;
+                                    cch += pfnOutput(pvArgOutput, &ch, 1);
 #endif
+                                }
+                                while (--cchWidth >= 0)
+                                    cch += pfnOutput(pvArgOutput, " ", 1);
                             }
-                            while (--cchWidth >= 0)
-                                cch += pfnOutput(pvArgOutput, " ", 1);
+                            else
+                                cch = rtStrFormatBadPointer(cch, pfnOutput, pvArgOutput, cchWidth, fFlags,
+                                                            pwszStr, szTmp, RT_STR_TUPLE("!BadStrW"));
                         }
                         else if (chArgSize == 'L')
                         {
                             /* unicp -> utf8 */
-                            int         cchStr;
-                            PCRTUNICP   puszStr = va_arg(args, PCRTUNICP);
-
-                            if (!VALID_PTR(puszStr))
+                            PCRTUNICP puszStr = va_arg(args, PCRTUNICP);
+                            if (RT_VALID_PTR(puszStr))
                             {
-                                static RTUNICP s_uszNull[] = {'<', 'N', 'U', 'L', 'L', '>', '\0' };
-                                puszStr = s_uszNull;
-                            }
-                            cchStr = _strnlenUni(puszStr, (unsigned)cchPrecision);
-                            if (!(fFlags & RTSTR_F_LEFT))
-                                while (--cchWidth >= cchStr)
-                                    cch += pfnOutput(pvArgOutput, " ", 1);
+                                int cchStr = rtStrFormatStrNLenUni(puszStr, (unsigned)cchPrecision);
+                                if (!(fFlags & RTSTR_F_LEFT))
+                                    while (--cchWidth >= cchStr)
+                                        cch += pfnOutput(pvArgOutput, " ", 1);
 
-                            cchWidth -= cchStr;
-                            while (cchStr-- > 0)
-                            {
+                                cchWidth -= cchStr;
+                                while (cchStr-- > 0)
+                                {
 /** @todo \#ifndef IN_RC*/
 #ifdef IN_RING3
-                                char *pszEnd = RTStrPutCp(szTmp, *puszStr++);
-                                cch += pfnOutput(pvArgOutput, szTmp, pszEnd - szTmp);
+                                    char *pszEnd = RTStrPutCp(szTmp, *puszStr++);
+                                    cch += pfnOutput(pvArgOutput, szTmp, pszEnd - szTmp);
 #else
-                                char ch = (char)*puszStr++;
-                                cch += pfnOutput(pvArgOutput, &ch, 1);
+                                    char ch = (char)*puszStr++;
+                                    cch += pfnOutput(pvArgOutput, &ch, 1);
 #endif
+                                }
+                                while (--cchWidth >= 0)
+                                    cch += pfnOutput(pvArgOutput, " ", 1);
                             }
-                            while (--cchWidth >= 0)
-                                cch += pfnOutput(pvArgOutput, " ", 1);
+                            else
+                                cch = rtStrFormatBadPointer(cch, pfnOutput, pvArgOutput, cchWidth, fFlags,
+                                                            puszStr, szTmp, RT_STR_TUPLE("!BadStrU"));
                         }
                         else
                         {
-                            int   cchStr;
-                            const char *pszStr = va_arg(args, char*);
+                            const char *pszStr = va_arg(args, const char *);
+                            if (RT_VALID_PTR(pszStr))
+                            {
+                                int cchStr = rtStrFormatStrNLen(pszStr, (unsigned)cchPrecision);
+                                if (!(fFlags & RTSTR_F_LEFT))
+                                    while (--cchWidth >= cchStr)
+                                        cch += pfnOutput(pvArgOutput, " ", 1);
+
+                                cch += pfnOutput(pvArgOutput, pszStr, cchStr);
 
-                            if (!VALID_PTR(pszStr))
-                                pszStr = "<NULL>";
-                            cchStr = _strnlen(pszStr, (unsigned)cchPrecision);
-                            if (!(fFlags & RTSTR_F_LEFT))
                                 while (--cchWidth >= cchStr)
                                     cch += pfnOutput(pvArgOutput, " ", 1);
-
-                            cch += pfnOutput(pvArgOutput, pszStr, cchStr);
-
-                            while (--cchWidth >= cchStr)
-                                cch += pfnOutput(pvArgOutput, " ", 1);
+                            }
+                            else
+                                cch = rtStrFormatBadPointer(cch, pfnOutput, pvArgOutput, cchWidth, fFlags,
+                                                            pszStr, szTmp, RT_STR_TUPLE("!BadStr"));
                         }
                         break;
                     }
@@ -726,8 +766,56 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                                 fFlags |= RTSTR_GET_BIT_FLAG(unsigned int);
                             }
                         }
-                        cchNum = RTStrFormatNumber((char *)&szTmp, u64Value, uBase, cchWidth, cchPrecision, fFlags);
-                        cch += pfnOutput(pvArgOutput, (char *)&szTmp, cchNum);
+                        cchNum = RTStrFormatNumber(&szTmp[0], u64Value, uBase, cchWidth, cchPrecision, fFlags);
+                        cch += pfnOutput(pvArgOutput, &szTmp[0], cchNum);
+                        break;
+                    }
+
+                    /*
+                     * Floating point.
+                     *
+                     * We currently don't really implement these yet, there is just a very basic
+                     * formatting regardless of the requested type.
+                     */
+                    case 'e': /* [-]d.dddddde+-dd[d] */
+                    case 'E': /* [-]d.ddddddE+-dd[d] */
+                    case 'f': /* [-]dddd.dddddd / inf / nan */
+                    case 'F': /* [-]dddd.dddddd / INF / NAN */
+                    case 'g': /* Either f or e, depending on the magnitue and precision. */
+                    case 'G': /* Either f or E, depending on the magnitue and precision. */
+                    case 'a': /* [-]0xh.hhhhhhp+-dd */
+                    case 'A': /* [-]0Xh.hhhhhhP+-dd */
+                    {
+#if defined(IN_RING3) && !defined(IN_SUP_HARDENED_R3) && !defined(IPRT_NO_FLOAT_FORMATTING)
+                        size_t cchNum;
+# ifdef RT_COMPILER_WITH_80BIT_LONG_DOUBLE
+                        if (chArgSize == 'L')
+                        {
+                            RTFLOAT80U2 r80;
+                            r80.lrd = va_arg(args, long double);
+#  ifndef IN_BLD_PROG
+                            cchNum = RTStrFormatR80u2(&szTmp[0], sizeof(szTmp), &r80, cchWidth, cchPrecision, 0);
+#  else
+                            cch += pfnOutput(pvArgOutput, RT_STR_TUPLE("<long double>"));
+                            RT_NOREF_PV(r80);
+                            break;
+#  endif
+                        }
+                        else
+# endif
+                        {
+                            RTFLOAT64U r64;
+                            r64.rd = va_arg(args, double);
+# ifndef IN_BLD_PROG
+                            cchNum = RTStrFormatR64(&szTmp[0], sizeof(szTmp), &r64, cchWidth, cchPrecision, 0);
+# else
+                            cchNum = RTStrFormatNumber(&szTmp[0], r64.au64[0], 16, 0, 0, RTSTR_F_SPECIAL | RTSTR_F_64BIT);
+# endif
+                        }
+                        cch += pfnOutput(pvArgOutput, &szTmp[0], cchNum);
+#else  /* !IN_RING3 */
+                        AssertFailed();
+#endif /* !IN_RING3 */
                         break;
                     }
 
